/*
 * @Author: kael
 * @Date: 2017-04-07 10:54:48
 * @Last Modified by: kael
 * @Last Modified time: 2017-04-13 09:55:54
 */

const fs = require('fs');
const path = require('path');
const Router = require('koa-router');
const walkSync = require('walk-sync');
const debug = require('debug')('kkk:router');

// router alias
// example: @alias('/alias/params');
const ALIAS_REG_G = /(?:\/\/@alias\(')([^']+)(?:'\);?router\.\w+\(')([^']+)(?:')/g;
const ALIAS_REG = /(?:\/\/@alias\(')([^']+)(?:'\);?router\.\w+\(')([^']+)(?:')/;
const ALIAS_MATCHS = {};

function alias_parse(file, prefix) {
  fs.readFile(file, (err, data) => {
    if (err) return debug(`alias parse error: ${file}`);
    let content = data.toString().replace(/\s+/g, '').replace(/["`]+/g, `'`);
    (content.match(ALIAS_REG_G) || []).forEach(match => {
      let temp = match.match(ALIAS_REG);
      let router_uri = ALIAS_MATCHS[temp[1]] = `${prefix}${temp[2]}`;
      debug(`router alias: ${temp[1]} -> ${router_uri}`);
    });
  });
}

function alias_check(ctx = {}) {
  let alias_path = ALIAS_MATCHS[ctx.path];
  if (alias_path) {
    ctx.path = alias_path;
  }
}

function load(dir, ex = '') {
  debug(`load router: ${dir}`);
  const routers = [];
  walkSync(dir, {
    globs: ['**/*.js'],
    directories: false,
  }).forEach(file => {
    let prefix = ex + '/' + file.replace(/\.js$/i, '');
    let router_module = path.join(dir, file);
    let router = new Router({
      prefix: prefix,
    });
    try {
      require(router_module)(router);
      routers.push(router.routes());
      alias_parse(router_module, prefix);
    } catch (ex) {
      debug(`load router module error: ${router_module}`);
      throw ex;
    }
  });
  return routers;
}

module.exports = function (app, dir, prefix) {
  // router init
  const routers = load(path.join(
    path.dirname(process.mainModule.filename), // main module root
    path.normalize(dir).replace(/\\/g, '/') // directory
  ), prefix);

  // router alias
  app.use(async (ctx, next) => {
    alias_check(ctx);
    await next();
  }, 'router_alias');

  // router use
  routers.forEach(midware => {
    app.use(midware);
  });

  return app;
};
